﻿using System;
using System.Globalization;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml.Linq;

namespace IconFixer
{
    internal class IconFixer
    {
        // contentclass class crawled property id
        public static readonly Guid crawledPropertyContentClass = new Guid("00140329-0000-0140-c000-000000141446");

        /// <summary>
        ///  determine if string is numeric or not
        /// </summary>
        /// <param name="Expression"></param>
        /// <returns></returns>
        private static Boolean IsNumeric (string Expression)
        {  
            try 
            {
                if(Expression is string)
                    Double.Parse(Expression as string);
                else
                    Double.Parse(Expression.ToString());
                    return true;
            } 
            catch {} // just dismiss errors but return false                
            return false;
        }
        
        /// <summary>
        ///  determine if contentclass belongs to custom doclib
        ///  if contentclass is (for example) STS_ListItem_10210 then make it custom
        ///  
        /// </summary>
        /// <param name="contentClass"></param>
        /// <returns></returns>
        private static Boolean IsCustomLibrary(string contentClass)
        {
            try {
                if (contentClass.Contains("STS_ListItem_"))
                {
                    char[] splitChar = { '_' };
                    string[] splittedContentClass = contentClass.Split(splitChar);
                    // need to be numeric. 
                    // need to be atleast 4 characters
                    if (IsNumeric(splittedContentClass.Last().ToString()) && splittedContentClass.Last().Length >= 4)
                    {
                        return true;
                    }
                    else return false;
                }
            }
            catch {} // just dismiss errors but return false
            return false;
        }

        // actual processing
        public void DoProcessing(string inputFile, string outputFile)
        {
            // use for debugging, otherwise impossible to attach to process
            // System.Threading.Thread.Sleep(50000); 

            XDocument inputElement = XDocument.Load(inputFile);
            // fetch the contentclass value from the input item with Linq2Xml
            var result = from cp in inputElement.Descendants("CrawledProperty")
                      where new Guid(cp.Attribute("propertySet").Value).Equals(crawledPropertyContentClass) &&
                      cp.Attribute("propertyName").Value == "DAV:contentclass" &&
                      cp.Attribute("varType").Value == "31" select cp.Value;             // vartype 31 is wide string
   
            if (result.Count() > 0 && result.First().Length > 0)
            {
                string inputValue = result.First();
                string outputValue;

                // if library is custom, add ootb contentclass to output element
                if (IsCustomLibrary(inputValue))
                {
                    outputValue = "STS_ListItem_DocumentLibrary";
                    XElement outputElement = GetOutputValue(inputValue, outputValue);
                    outputElement.Save(outputFile);         
                }
                else
                {
                    inputElement.Save(outputFile);
                }
            }
        }

        private static XElement GetOutputValue(string inputValue, string outputValue)
        {
            // create the output item
            XElement outputElement = new XElement("Document");
            if (inputValue.Count() > 0)
            {
                outputElement.Add(new XElement("CrawledProperty",
                                  new XAttribute("propertySet", crawledPropertyContentClass),
                                  new XAttribute("propertyName", "DAV:contentclass"),
                                  new XAttribute("varType", 31), outputValue)
                    );
            }
            return outputElement;
        }
    }
}