﻿using System;
using System.Threading;
using System.IO;
using System.Text;
using System.Reflection;

namespace IconFixer
{    
    /// <author>Bastiaan Kortenbout</author>
    /// <summary>
    /// this is workaround to fix missing icons in search results for documents that exist in custom doclibs (list id != 101)
    /// bug found by neil richards http://www.neilrichards.net/blog/?p=184
    /// copied some stuff from:
    /// https://blogs.msdn.com/b/thomsven/archive/2010/09/23/debugging-and-tracing-fast-search-pipeline-extensibility-stages.aspx
    /// https://www.nothingbutsharepoint.com/sites/devwiki/articles/Pages/How-to-Create-and-debug-a-FAST-Search-for-SharePoint-pipeline-extensibility-stage-in-C-Sharp-and-Powershell.aspx
    /// </summary>
    internal class Program
    {        
        // timestamp for logging purposes
        public static readonly String timestamp = DateTime.Now.ToString("yyyyMMddHHmmss.ffff");

        /// <summary>
        ///  write the input/output file to a location the application has access to write in
        /// </summary>
        /// <param name="inputFile"></param>
        /// <param name="suffix"></param>      
        static void WriteLogFile(string inputFile, string suffix)        
        {            
            String pipelineInputData = @"c:\users\" + Environment.UserName + @"\appdata\LocalLow\PipelineLog";             
            // enable/disable debugging in real-time by creating/renaming the log directory            
            if (Directory.Exists(pipelineInputData))            
            {       
                string outFile = Path.Combine(pipelineInputData, timestamp + "-" + MethodBase.GetCurrentMethod().DeclaringType.Name + suffix + ".xml");                
                File.Copy(inputFile, outFile);            
            }        
        }

        /// <summary>
        /// log stuff and start iconfixer
        /// </summary>
        /// <param name="args"></param>
        /// <returns></returns>
        private static int Main(string[] args)
        {
            try
            {
                String inputXmlFile = args[0]; 
                String outputXmlFile = args[1]; 
                
                // log and kick off
                WriteLogFile(inputXmlFile, "-input");
                IconFixer iconFixer = new IconFixer();
                iconFixer.DoProcessing(inputXmlFile, outputXmlFile);
                WriteLogFile(outputXmlFile, "-output"); 
            }
            catch (Exception e)
            {
                // this will end up in the crawl log, since exit code != 0
                Console.WriteLine("Failed: " + e.Message + "/" + e.StackTrace);
                return 1;
            }
            return 0;
        }
    }
}